package ActiveState::MIME::Parser;

use strict;

our $VERSION = '1.00';

use XSLoader ();
XSLoader::load(__PACKAGE__, $VERSION);

sub new {
    my $class = shift;
    my $self = bless {}, $class;
    _alloc_mp($self);
    $self;
}

# legacy
*decode_multipart = \&descend_multipart;

1;

__END__

=head1 NAME

ActiveState::MIME::Parser - Event based parser for MIME messages

=head1 SYNOPSIS

  use ActiveState::MIME::Parser;

  $p = ActiveState::MIME::Parser->new;
  $p->handler(part_start => \&part_start);
  $p->handler(part_end   => \&part_end);
  $p->handler(data_raw   => \&data_h);

  # Parse message chunk by chunk
  $p->parse($chunk1);
  $p->parse($chunk2);
  #...
  $p->eof; # signal end of document

  # Parse directly from file
  $p->parse_file("message");

=head1 DESCRIPTION

Instances of the C<ActiveState::MIME::Parser> class can be used to break up
and decode messages that follow the RFC822/MIME format used for
Internet e-mail and USENET News.  MIME is described in RFC 2045 and RFC 2046.
The RFC822 format is described in RFC 2822.

In contrast to the CPAN module C<MIME::Parser> this module has a much
simpler API (and implementation), it does not require stream objects
to parse from, it is totally event based, and there are never any
temporary files created behind the scenes.  The difference is in many
ways similar to the difference between SAX and DOM parsers in the XML
world.

A message to be parsed is given to the C<ActiveState::MIME::Parser> in
arbitrary chunks by calling the parse() method as many times as
needed.  The end of message is then signaled by calling eof().  The
parser object will as a result of the data it is feed invoke
registered callback handlers.  The callbacks are invoked as early as
possbile to minimize the need for internal buffering in the parser
object.

The parser object is a hash and can be used to store state during
parsing.  Hash keys prefixed with "_mp_" are reserved for the parser
itself.

The following methods are provided:

=over

=item $p = ActiveState::MIME::Parser->new;

The constructor method is used to create new instances of the
C<ActiveState::MIME::Parser> class.  Currently it does not take any
arguments.  It will be extended so that initial handlers and parser
attribute values could be set up here.

=item $p->handler( event => \&subroutine )

=item $p->handler( event => method_name )

=item $p->handler( event => undef )

=item $p->handler( event )

The event method is used to register event handlers with the parser.
There are currently 5 kinds of events that can be reported:

  part_start($p, $hdrs, $ct, \%ct_param, $te)
  part_end($p)
  data_raw($p, $rdata)
  data_decoded($p, $ddata)
  boundary($p, $bdata)

All event handlers receive a reference to the parser object ($p) as
their first argument.

The C<part_start> event is called with the all the headers fields as
the first additional argument.  This is a single string containing all
the headers.  Then the parsed out and normalized C<Content-Type> and
any extra parameters that went with it as well as the normlized
C<Content-Transfer-Encoding> value.  The top level message itself is also
reported as a C<part_start> event.

The C<part_end> event does not receive any extra arguments.  It is
called to signal the end of a part.  The C<part_start> and C<part_end>
pairs will always be called such that they nest properly.

The C<data_raw> and C<data_decoded> events are called to report body
data inside a part.  The extra argument is a chunk of data.  No
guaranties are made about how this data is divided up between chunks,
but the $data argument will never be empty.

The C<boundary> event is called to report any boundary markers during
processing.  It is only useful to applications that want to make sure
they filter a message with the minimal amount of changes, as this
allow the application to preserve trailing whitespace and notice if
ending boundaries happen to be missing.

The concatenation of all the $hdrs, $rdata and $bdata argumented
reported will give exactly the original message.

The return value of handler callbacks are ignored.

The return value of the handler method is the old callback value.  The
callback value can be set to C<undef>.  The effect is the same as
setting it to C<sub {}>, but it is a more efficient way to ignore
events.

=item $p->parse( $string )

Parse $string as the next chunk of the message.

=item $p->eof

When all the chunks of the message has been reported with the parse()
method, then the eof() needs to be invoked to flush any buffered data
and also invoke the final C<end_part> events.

=item $p->parse_file( $filename )

This is a convenience method that parse data from the file given as
argument.  Currently the argument must be a file name, but it will be
extended to also allow already opened file handlers in the future.

=item $p->part_id

This reports a short string that identifies this part in the the
multipart hierarchy.  The top level message is identified as "1".  The
first part in the top level message is "1.1" and the second is "1.2".
The first part inside the second part of the top level message is
"1.2.1".

=item $p->offset

This reports the current offset into the message.  When called from
within a event callback, the offset is the location of the start of
the data corresponding to the current event.  For C<part_end> events
it would be the offset where the data of that parts ends.

=item $p->descend_multipart

=item $p->descend_multipart( $bool )

This boolean attribute will if TRUE instruct the parser to traverse
down into nested body parts.  The default is TRUE.

If it is FALSE, then the part body will be reported as data
instead.

This attribute was called I<decode_multipart> in older versions of
this module and that name still works.

=item $p->descend_sub_message

=item $p->descend_sub_message( $bool )

This boolean attribute will if TRUE instruct the parser to traverse
down into nested message/rfc822 parts.  The default is FALSE.

=item $p->allow_from

=item $p->allow_from( $bool )

This boolean attribute will if TRUE instruct the parser to tolerate an
mbox style "From "-line as the first before the headers.  The default
is TRUE.

=back

=head1 BUGS

Should be support to descend into uuencoded stuff as well as tar and zip
files.

Should be support for pushing additional decoders in the C<part_start>
callback.  Could for instance be used to deal with automatic
decompression.

=head1 COPYRIGHT

Copyright (c) 2003 ActiveState Corp.  All rights reserved.

=head1 SEE ALSO

L<MIME::Parser>

=cut
